//
// CmnNet.h - Network support
// Date :	27.7.2003
// Update :	27.7.2003
// Author :	Ondrej Burisin, Jiri Formanek
//
#include "stdafx.h"
#define COMMON_API __declspec(dllexport)
#include "CmnLog.h"
#include "CmnNet.h"

COMMON_API DWORD	InitNetwork()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	wVersionRequested = MAKEWORD( 1, 1 );
	if ( WSAStartup(wVersionRequested, &wsaData) != 0 ) {
		// Tell the user that we could not find a usable 
		ERR("Cannot init WinSock\n", WSAGetLastError());
		return WSAGetLastError();
	}
	return S_OK;
}

COMMON_API DWORD	DestroyNetwork()
{
	 return (DWORD)WSACleanup();
}

// interface functions
HRESULT		XBlockingSocket::AddRef()
{	
	return m_dwRef++;
}

HRESULT		XBlockingSocket::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{
		delete this;
	}
	return m_dwRef;
}

void XBlockingSocket::Cleanup()
{
	// doesn't throw an exception because it's called in a catch block
	if(m_hSocket == NULL) return;
	closesocket(m_hSocket);
	m_hSocket = NULL;
}

void XBlockingSocket::Create(int nType /* = SOCK_STREAM */)
{	
	if((m_hSocket = socket(AF_INET, nType, 0)) == INVALID_SOCKET) {
		XException exp("Create()", WSAGetLastError());
		THROW(exp);
	}
}

void XBlockingSocket::Bind(LPCSOCKADDR psa)
{
	if(bind(m_hSocket, psa, sizeof(SOCKADDR)) == SOCKET_ERROR) {
		XException exp("Bind()", WSAGetLastError());
		THROW(exp);		
	}
}

void XBlockingSocket::Listen()
{
	if(listen(m_hSocket, 5) == SOCKET_ERROR) {
		XException exp("Listen()", WSAGetLastError());
		THROW(exp);		
	}
}

BOOL XBlockingSocket::Accept(IBlockingSocket& sConnect, LPSOCKADDR psa)
{
	int nLengthAddr = sizeof(SOCKADDR);
	sConnect.Create(accept(m_hSocket, psa, &nLengthAddr));
	if(sConnect == INVALID_SOCKET) {
		// no exception if the listen was canceled
		if(WSAGetLastError() != WSAEINTR) {
			XException exp("Accept()", WSAGetLastError());
			THROW(exp);		
		}
		return FALSE;
	}
	return TRUE;
}

void XBlockingSocket::Close()
{	
	if(closesocket(m_hSocket) == SOCKET_ERROR) {
		// should be OK to close if closed already
		XException exp("Close()", WSAGetLastError());
		THROW(exp);		
	}
	m_hSocket = NULL;
}

void XBlockingSocket::Connect(LPCSOCKADDR psa)
{	
	// should timeout by itself
	if(connect(m_hSocket, psa, sizeof(SOCKADDR)) == SOCKET_ERROR) {		
		XException exp("Connect()", WSAGetLastError());
		THROW(exp);
	}
}

int XBlockingSocket::Write(const char* pch, const int nSize, const int nSecs)
{
	int nBytesSent = 0;
	int nBytesThisTime;
	const char* pch1 = pch;
	do {
		nBytesThisTime = Send(pch1, nSize - nBytesSent, nSecs);
		nBytesSent += nBytesThisTime;
		pch1 += nBytesThisTime;
	} while(nBytesSent < nSize);
	return nBytesSent;
}

int XBlockingSocket::Send(const char* pch, const int nSize, const int nSecs)
{
	// returned value will be less than nSize if client cancels the reading
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, NULL, &fd, NULL, &tv) == 0) {
		XException exp("Send timeout", WSAGetLastError());
		THROW(exp);		
	}
	int nBytesSent;
	if((nBytesSent = send(m_hSocket, pch, nSize, 0)) == SOCKET_ERROR) {
		XException exp("Send()", WSAGetLastError());
		THROW(exp);		
	}
	return nBytesSent;
}

int XBlockingSocket::Receive(char* pch, const int nSize, const int nSecs)
{	
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, &fd, NULL, NULL, &tv) == 0) {
		XException exp("Recieve timeout", WSAGetLastError());
		THROW(exp);		
	}

	int nBytesReceived;
	if((nBytesReceived = recv(m_hSocket, pch, nSize, 0)) == SOCKET_ERROR) {
		XException exp("Recieve()", WSAGetLastError());
		THROW(exp);		
	}
	return nBytesReceived;
}

int XBlockingSocket::ReceiveDatagram(char* pch, const int nSize, LPSOCKADDR psa, const int nSecs)
{
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, &fd, NULL, NULL, &tv) == 0) {
		XException exp("Recieve timeout", WSAGetLastError());
		THROW(exp);		
	}

	// input buffer should be big enough for the entire datagram
	int nFromSize = sizeof(SOCKADDR);
	int nBytesReceived = recvfrom(m_hSocket, pch, nSize, 0, psa, &nFromSize);
	if(nBytesReceived == SOCKET_ERROR) {
		XException exp("RecieveDatagram()", WSAGetLastError());
		THROW(exp);		
	}
	return nBytesReceived;
}

int XBlockingSocket::SendDatagram(const char* pch, const int nSize, LPCSOCKADDR psa, const int nSecs)
{
	FD_SET fd = {1, m_hSocket};
	TIMEVAL tv = {nSecs, 0};
	if(select(0, NULL, &fd, NULL, &tv) == 0) {
		XException exp("Send timeout", WSAGetLastError());
		THROW(exp);		
	}

	int nBytesSent = sendto(m_hSocket, pch, nSize, 0, psa, sizeof(SOCKADDR));
	if(nBytesSent == SOCKET_ERROR) {
		XException exp("SendDatagram()", WSAGetLastError());
		THROW(exp);		
	}
	return nBytesSent;
}

void XBlockingSocket::GetPeerAddr(LPSOCKADDR psa)
{	
	// gets the address of the socket at the other end
	int nLengthAddr = sizeof(SOCKADDR);
	if(getpeername(m_hSocket, psa, &nLengthAddr) == SOCKET_ERROR) {
		XException exp("GetPeerName()", WSAGetLastError());
		THROW(exp);		
	}
}

void XBlockingSocket::GetSockAddr(LPSOCKADDR psa)
{
	//ASSERT(m_hSocket != NULL);
	// gets the address of the socket at this end
	int nLengthAddr = sizeof(SOCKADDR);
	if(getsockname(m_hSocket, psa, &nLengthAddr) == SOCKET_ERROR) {		
		XException exp("GetSockAddr()", WSAGetLastError());
		THROW(exp);
	}
}

//static
SockAddr XBlockingSocket::GetHostByName(const char* pchName, const USHORT ushPort /* = 0 */)
{
	hostent* pHostEnt = gethostbyname(pchName);
	if(pHostEnt == NULL) {		
		XException exp("GetHostByName()", WSAGetLastError());
		THROW(exp);
	}		
	ULONG* pulAddr = (ULONG*) pHostEnt->h_addr_list[0];
	SOCKADDR_IN sockTemp;
	sockTemp.sin_family = AF_INET;
	sockTemp.sin_port = htons(ushPort);
	sockTemp.sin_addr.s_addr = *pulAddr; // address is already in network byte order
	return sockTemp;
}

//static
const char* XBlockingSocket::GetHostByAddr(LPCSOCKADDR psa)
{
	hostent* pHostEnt = gethostbyaddr((char*) &((LPSOCKADDR_IN) psa)
				->sin_addr.s_addr, 4, PF_INET);
	if(pHostEnt == NULL) {
		XException exp("GetHostByAddr()", WSAGetLastError());
		THROW(exp);		
	}		
	return pHostEnt->h_name; // caller shouldn't delete this memory
}
