#include "StdAfx.h"

#define COMMON_API __declspec(dllexport)

#include "CmnLog.h"
#include "CmnError.h"
#include "CmnZip.h"
#include "CmnStr.h"
#include "CmnFile.h"

compress_func	compress;
uncompress_func	uncompress;

DWORD	InitZipLib() throw( XException )
{
	HMODULE	hZipLibrary = NULL;	
	DWORD	dwRet = ERROR_SUCCESS;
	/* try load zip library with UNICODE version of  LoadLibrary()*/
	if(!hZipLibrary) {		
		wchar_t	wszPath[MAX_PATH];		
		/* bild path for library */
		cmnGetDataFilePathW(wszPath, _S_ZIP_DLLW);
		/* load library */
		hZipLibrary = LoadLibraryW(wszPath);
		if(!hZipLibrary) {		
			dwRet = GetLastError();
			/* if library not found, GetLastError() returns ERROR_FILE_NOT_FOUND 
			 -> dont try ANSI */
			if(dwRet == ERROR_MOD_NOT_FOUND) {
				return dwRet;
			}
		}		
	}
	/* if hZipLibrary is still NULL, UNICODE cannot be used, so try ANSI version of  LoadLibrary()*/
	if(!hZipLibrary) {		
		char	szPath[MAX_PATH];	
		/* bild path for library */
		cmnGetDataFilePathA(szPath, _S_ZIP_DLLA);	
		/* load library */
		hZipLibrary = LoadLibraryA(szPath);
		if(!hZipLibrary) {			
			return GetLastError();
		}		
	}
	/* if hZipLibrary is valid, try to get pointers to desired functions */
	if(hZipLibrary) {
		/* compress function */
		compress = (compress_func)GetProcAddress(hZipLibrary, FUNC_COMPRESS);
		/* uncompress function */
		uncompress = (compress_func)GetProcAddress(hZipLibrary, FUNC_UNCOMPRESS);
		if(compress == NULL || uncompress == NULL) {
			XException exp("Cannot get proc adress from library.", GetLastError());
			THROW(exp);			
		}
		TRACE(_T("Zip support is available."));
	}
	return dwRet;
}

// compress specified src buffer to dest buffer 
COMMON_API DWORD	cmnCompressBuffer(LPBYTE lpDest, LPDWORD lpdwDestLen, LPBYTE lpSrc, DWORD dwSrcLen)
{
	DWORD dwRet;
	int zErr;
	if(uncompress) {
		/* zErr can be negative value */
		zErr = compress(lpDest, lpdwDestLen, lpSrc, dwSrcLen); 
		/* convert return code of compress function to standard codes */
		switch(zErr) {
			case Z_OK:
				dwRet = ERROR_SUCCESS;
				break;
			case Z_MEM_ERROR:
				dwRet = ERROR_NOT_ENOUGH_MEMORY;
				break;
			case Z_BUF_ERROR:
				dwRet = ERROR_INSUFFICIENT_BUFFER;
				break;
			default:
				dwRet = E_CMN_UNKNOWN;
				break;
		}
	}
	else {
		dwRet = ERROR_CALL_NOT_IMPLEMENTED;
	}
	return dwRet;
}

// uncompress specified src buffer to dest buffer
COMMON_API DWORD	cmnUncompressBuffer(LPBYTE lpDest, LPDWORD lpdwDestLen, LPBYTE lpSrc, DWORD dwSrcLen)
{
	DWORD dwRet;
	int zErr;
	if(uncompress) {
		/* zErr can be negative value */
		zErr = uncompress(lpDest, lpdwDestLen, lpSrc, dwSrcLen); 
		/* convert return code of uncompress function to standard codes */
		switch(zErr) {
			case Z_OK:
				dwRet = ERROR_SUCCESS;
				break;
			case Z_MEM_ERROR:
				dwRet = ERROR_NOT_ENOUGH_MEMORY;
				break;
			case Z_BUF_ERROR:
				dwRet = ERROR_INSUFFICIENT_BUFFER;
				break;
			case Z_DATA_ERROR:
				dwRet = ERROR_INVALID_DATA;
				break;
			default:
				dwRet = E_CMN_UNKNOWN;
				break;
		}
	}
	else {
		dwRet = ERROR_CALL_NOT_IMPLEMENTED;
	}
	return dwRet;
}
	
/*
 *	Uncompress file previously compressed by CompressFile() function,
 *
 *	csPath		- input file path
 *	lplpData	- is address of output buffer. If it is NULL uncompressed data
 *	are written to same file.
 *	pdwSize		- output size of uncompressed buffer, it may be NULL.
 *
 */
DWORD	cmnUncompressFileW(LPCWSTR lpwszPath, LPVOID* lplpData, DWORD *pdwSize, LPCOMPRESSIONINFO  pCompInfo)
{
	DWORD dwRet = 1;
	HANDLE	hFile;
	DWORD	dwUCSize, dwCSize;
	LPBYTE	lpUCData, lpCData;
	DWORD	dwReaden, dwWritten;
	//
	// Try to open specified file for reading
	hFile = cmnOpenExistingFileW(lpwszPath, GENERIC_READ);	
	if(hFile == INVALID_HANDLE_VALUE) {		
		XException exp("Cannot open file reading.",  GetLastError());
		THROW(exp);
	}
	//
	// Read compress and uncompress sizes
	if(!ReadFile(hFile, &dwCSize, sizeof(DWORD), &dwReaden, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot load C size due ", dwRet);
		return dwRet;
	}
	if(!ReadFile(hFile, &dwUCSize, sizeof(DWORD), &dwReaden, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot load UC size due ", dwRet);
		return dwRet;
	}	
	//
	// Allocated buffers
	lpUCData = new BYTE[dwUCSize];
	lpCData  = new BYTE[dwCSize];
	//
	// Read compressed file to the buffer
	if(!ReadFile(hFile, lpCData, dwCSize, &dwReaden, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot load compressed data due ", dwRet);
		SAFE_DELETE_ARRAY(lpCData);
		SAFE_DELETE_ARRAY(lpUCData);
		return dwRet;
	}
	//
	// Close file
	CloseHandle(hFile);
	//
	// Uncompress file to buffer
	dwRet = cmnUncompressBuffer(lpUCData, &dwUCSize, lpCData, dwCSize);	
	if(dwRet != ERROR_SUCCESS) {
		ERRW(L"Cannot uncompress file due ", dwRet);
		return dwRet;
	}
	//
	// Init information structure about compression
	if(pCompInfo) {
		pCompInfo->dwCompressedSize = dwCSize;
		pCompInfo->dwUncompressedSize = dwUCSize;
		pCompInfo->fRatio = (float(dwCSize) / float(dwUCSize)) * 100.0f;
	}
	//
	// Release compressed buffer
	SAFE_DELETE_ARRAY(lpCData);
	//
	// Write new file if input buffer is NULL
	if(lplpData == NULL) {
		//
		// Create new file for writting
		hFile = cmnCreateFileW(lpwszPath, GENERIC_WRITE);
		if(hFile == INVALID_HANDLE_VALUE) {
			dwRet = GetLastError();
			ERRW(L"Cannot open file for writting due ", dwRet);
			return dwRet;
		}
		//
		// Write uncompress data
		if(!WriteFile(hFile, lpUCData, dwUCSize, &dwWritten, NULL)) {
			dwRet = GetLastError();
			ERRW(L"Cannot write uncompressed data to file due ", dwRet);
			return dwRet;
		}
		//
		// Close file
		CloseHandle(hFile);
	}
	// Return UC file via output buffer
	else {
		//
		// Allocate memory for output buffer
		(*lplpData) = new BYTE[dwUCSize];
		//
		// Copy data to new buffer
		memcpy((*lplpData), lpUCData, dwUCSize);
		//
		// Set output size
		(*pdwSize) = dwUCSize;
	}		
	// Release uncompressed buffer
	SAFE_DELETE_ARRAY(lpUCData);
	//
	return dwRet;
}

/*
 *	Compress specified file
 *	
 *	Destroy previous file and create new file that is compressed
 *	As first in this file is Uncompressed size in DWORD
 *	Then is saved compressed size in DWORD.
 *	Rest of file is compressed file;
 *
 *	Input file must exist and must be closed
 *
 */
DWORD	cmnCompressFileW(LPCWSTR lpwszPath, LPCOMPRESSIONINFO  pCompInfo)
{
	DWORD dwRet = 1;
	HANDLE	hFile;
	DWORD	dwUCSize, dwCSize;
	LPBYTE	lpUCData, lpCData;
	DWORD	dwReaden, dwWritten;
	//
	// Try to open specified file for reading
	hFile = cmnOpenExistingFileW(lpwszPath, GENERIC_READ);		
	if(hFile == INVALID_HANDLE_VALUE) {
		dwRet = GetLastError();
		ERRW(L"Cannot open file reading due", dwRet);
		return dwRet;
	}
	//
	// Get file lenght
	dwUCSize = GetFileSize(hFile, NULL);
	dwRet = GetLastError();
	if(dwRet != NO_ERROR) {
		ERRW(L"Cannot get file size due", dwRet);
		return dwRet;
	}
	//
	// Read all content of file to buffer
	// Create buffer
	lpUCData = new BYTE[dwUCSize];
	if(!lpUCData) {
		return ERROR_NOT_ENOUGH_MEMORY;
	}
	// Fill buffer
	if(!ReadFile(hFile, lpUCData, dwUCSize, &dwReaden, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot load file to buffer due", dwRet);
		return dwRet;
	}
	//
	// Close file
	if(cmnCloseFile(hFile)) {
		SAFE_DELETE_ARRAY(lpUCData);
		return ERROR_INVALID_PARAMETER;
	}
	//
	// Prepare compress buffer
	dwCSize = dwUCSize + DWORD(dwUCSize * 0.1) + 12;
	lpCData = new BYTE[dwCSize];
	if(!lpCData) {
		SAFE_DELETE_ARRAY(lpUCData);
		return ERROR_NOT_ENOUGH_MEMORY;
	}
	//
	// Compress
	dwRet = cmnCompressBuffer(lpCData, &dwCSize, lpUCData, dwUCSize);	
	if(dwRet != ERROR_SUCCESS) {
		ERRW(L"Cannot compress file due ", dwRet);
		return dwRet;
	}
	//
	// Release first buffer
	SAFE_DELETE_ARRAY(lpUCData);
	//
	// Try to open specified file for writting
	hFile = cmnCreateFileW(lpwszPath, GENERIC_WRITE);		
	if(hFile == INVALID_HANDLE_VALUE) {
		dwRet = GetLastError();
		ERRW(L"Cannot open file for writting due", dwRet);
		return dwRet;
	}
	//
	// Write compress and uncompress sizes	
	if(!WriteFile(hFile, &dwCSize, sizeof(DWORD), &dwWritten, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot write C size to file due", dwRet);
		return dwRet;
	}
	if(!WriteFile(hFile, &dwUCSize, sizeof(DWORD), &dwWritten, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot write UC size to file due", dwRet);
		return dwRet;
	}
	//
	// Write compress data
	if(!WriteFile(hFile, lpCData, dwCSize, &dwWritten, NULL)) {
		dwRet = GetLastError();
		ERRW(L"Cannot write compressed data to file due", dwRet);
		return dwRet;
	}
	//
	// Init information structure about compression
	if(pCompInfo) {
		pCompInfo->dwCompressedSize = dwCSize;
		pCompInfo->dwUncompressedSize = dwUCSize;
		pCompInfo->fRatio = (float(dwCSize) / float(dwUCSize)) * 100.0f;
	}
	// Release second buffer
	SAFE_DELETE_ARRAY(lpCData);
	//
	// Close file
	if(cmnCloseFile(hFile)) {
		return ERROR_INVALID_PARAMETER;
	}	
	return dwRet;
}


/*
 *	Function converts input ANSI string to UNICODE and call UNICODE version of 
 *	cmnCompressFile function
 */
COMMON_API DWORD	cmnCompressFileA(LPCSTR lpszPath, LPCOMPRESSIONINFO  pCompInfo)
{
	LPWSTR lpwszPath = NULL;
	DWORD dwRet;	
	DWORD dwLen = 0;
	//
	// Get length of future UNICODE string 
	dwRet = cmnConvertANSIToUNICODE(NULL, &dwLen, (LPSTR)lpszPath);	// auto length detection	
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	//
	//	Allocate memory for UNICODE string
	lpwszPath = new wchar_t[dwLen];
	if(lpwszPath)
	{
		// Convert ANSI to UNICODE
		dwRet = cmnConvertANSIToUNICODE(lpwszPath, &dwLen, (LPSTR)lpszPath);
		if(dwRet != ERROR_SUCCESS) {
			SAFE_DELETE_ARRAY(lpwszPath);
			return dwRet;
		}
		// Call UNICODE version of cmnCompressFile funcion
		dwRet = cmnCompressFileW(lpwszPath, pCompInfo);
		SAFE_DELETE_ARRAY(lpwszPath);
	}
	return dwRet;
}


/*
 *	Function converts input ANSI string to UNICODE and call UNICODE version of 
 *	cmnUncompressFile function
 */
COMMON_API DWORD	cmnUncompressFileA(LPCSTR lpszPath, LPVOID* lplpData, DWORD *pdwSize, LPCOMPRESSIONINFO  pCompInfo)
{
	LPWSTR lpwszPath = NULL;
	DWORD dwRet;	
	DWORD dwLen = 0;
	//
	// Get length of future UNICODE string 
	dwRet = cmnConvertANSIToUNICODE(NULL, &dwLen, (LPSTR)lpszPath);	// auto length detection
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	//
	//	Allocate memory for UNICODE string
	lpwszPath = new wchar_t[dwLen];
	if(lpwszPath)
	{
		// Convert ANSI to UNICODE
		dwRet = cmnConvertANSIToUNICODE(lpwszPath, &dwLen, (LPSTR)lpszPath);
		if(dwRet != ERROR_SUCCESS) {
			SAFE_DELETE_ARRAY(lpwszPath);
			return dwRet;
		}
		// Call UNICODE version of cmnCompressFile funcion
		dwRet = cmnUncompressFileW(lpwszPath, lplpData, pdwSize, pCompInfo);
		SAFE_DELETE_ARRAY(lpwszPath);
	}
	return dwRet;
}